package com.supervision.knowsub.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.lang.Assert;
import cn.hutool.core.util.StrUtil;
import com.supervision.knowsub.dto.flow.SystemFlowRuleDTO;
import com.supervision.knowsub.model.SystemFlowRule;
import com.supervision.knowsub.model.SystemFlowRuleUser;
import com.supervision.knowsub.service.FlowRuleManageService;
import com.supervision.knowsub.service.SystemFlowRuleService;
import com.supervision.knowsub.service.SystemFlowRuleUserService;
import com.supervision.knowsub.vo.flow.NodeInfo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class FlowRuleManageServiceImpl implements FlowRuleManageService {

    private final SystemFlowRuleService systemFlowRuleService;

    private final SystemFlowRuleUserService systemFlowRuleUserService;


    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveFlowRule(String flowId, List<NodeInfo> nodeInfoList) {
        Assert.notEmpty(flowId, "流程id不能为空");
        Assert.notEmpty(nodeInfoList, "节点信息不能为空");

        int i = 0;
        for (NodeInfo nodeInfo : nodeInfoList) {
            SystemFlowRule flowRule = new SystemFlowRule();
            flowRule.setFlowId(flowId);
            flowRule.setRuleName(nodeInfo.getNodeName());
            flowRule.setRuleOrder(i++);
            flowRule.setRoleId(nodeInfo.getRoleId());
            systemFlowRuleService.save(flowRule);

            systemFlowRuleUserService.saveBatch(nodeInfo.getUserIdList().stream().map(userId -> {
                SystemFlowRuleUser flowRuleUser = new SystemFlowRuleUser();
                flowRuleUser.setFlowId(flowId);
                flowRuleUser.setRuleId(flowRule.getId());
                flowRuleUser.setUserId(userId);
                return flowRuleUser;
            }).toList());
        }


        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveFlowRule(List<SystemFlowRuleDTO> systemFlowRuleDTOList) {
        if (CollUtil.isEmpty(systemFlowRuleDTOList)){
            return;
        }
        for (SystemFlowRuleDTO systemFlowRuleDTO : systemFlowRuleDTOList) {
            SystemFlowRule systemFlowRule = systemFlowRuleDTO.toSystemFlowRule();
            systemFlowRuleService.save(systemFlowRule);
            List<SystemFlowRuleUser> systemFlowRuleUserList = systemFlowRuleDTO.toSystemFlowRuleUserList();
            if (CollUtil.isNotEmpty(systemFlowRuleUserList)){
                for (SystemFlowRuleUser systemFlowRuleUser : systemFlowRuleUserList) {
                    systemFlowRuleUser.setRuleId(systemFlowRule.getId());
                    systemFlowRuleUserService.save(systemFlowRuleUser);
                }
            }
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateFlowRule(List<SystemFlowRuleDTO> systemFlowRuleDTOList) {
        if (CollUtil.isEmpty(systemFlowRuleDTOList)){
            return;
        }
        systemFlowRuleDTOList.forEach(systemFlowRuleDTO -> Assert.notEmpty(systemFlowRuleDTO.getId(), "流程id不能为空"));

        for (SystemFlowRuleDTO systemFlowRuleDTO : systemFlowRuleDTOList) {
            SystemFlowRule systemFlowRule = systemFlowRuleDTO.toSystemFlowRule();
            systemFlowRuleService.updateById(systemFlowRule);
            systemFlowRuleUserService.lambdaUpdate().eq(SystemFlowRuleUser::getRuleId, systemFlowRuleDTO.getId()).remove();
            List<SystemFlowRuleUser> systemFlowRuleUserList = systemFlowRuleDTO.toSystemFlowRuleUserList();
            if (CollUtil.isNotEmpty(systemFlowRuleUserList)){
                for (SystemFlowRuleUser systemFlowRuleUser : systemFlowRuleUserList) {
                    systemFlowRuleUser.setRuleId(systemFlowRule.getId());
                    systemFlowRuleUserService.save(systemFlowRuleUser);
                }
            }
        }
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateFlowRule(String flowId, List<NodeInfo> nodeInfoList) {
        Assert.notEmpty(flowId, "流程id不能为空");

        List<String> nodeIds = nodeInfoList.stream().map(NodeInfo::getId).filter(StrUtil::isNotEmpty).toList();

        List<SystemFlowRule> flowRuleList = systemFlowRuleService.lambdaQuery().eq(SystemFlowRule::getFlowId, flowId).list();
        List<String> flowRuleIdList = flowRuleList.stream().map(SystemFlowRule::getId).toList();
        Set<String> flowRuleIdSet = flowRuleList.stream().map(SystemFlowRule::getId).collect(Collectors.toSet());

        FlowRuleDTOPair flowRuleDTOPair = pickInsertAndUpdateFlowRuleDTO(flowId, nodeInfoList, flowRuleIdSet);

        // 新增数据
        if (CollUtil.isNotEmpty(flowRuleDTOPair.needAddRuleList())){
            saveFlowRule(flowRuleDTOPair.needAddRuleList());
        }

        // 更新数据
        if (CollUtil.isNotEmpty(flowRuleDTOPair.needUpdateRuleList())){
            updateFlowRule(flowRuleDTOPair.needUpdateRuleList());
        }
        //如果数据库中存在，列表中不存在，则删除
        List<String> needDeleteRuleIdList = flowRuleIdList.stream().filter(ruleId -> !nodeIds.contains(ruleId)).toList();
        if (CollUtil.isNotEmpty(needDeleteRuleIdList)){
            deleteFlowRuleByRuleIds(needDeleteRuleIdList);
        }
        return true;
    }


    /**
     * 提取需要新增和需要更新的数据
     * @param flowId 流程id
     * @param nodeInfoList 节点信息
     * @param flowRuleIdSet 数据库中已经存在的规则数据
     * @return
     */
    private FlowRuleDTOPair pickInsertAndUpdateFlowRuleDTO(String flowId, List<NodeInfo> nodeInfoList, Set<String> flowRuleIdSet) {
        List<SystemFlowRuleDTO> needUpdateRuleList = new ArrayList<>();
        List<SystemFlowRuleDTO> needAddRuleList = new ArrayList<>();
        int order = 0;
        for (NodeInfo nodeInfo : nodeInfoList) {
            nodeInfo.setOrder(order++);
            if (StrUtil.isNotEmpty(nodeInfo.getId()) && flowRuleIdSet.contains(nodeInfo.getId())){
                // 需要更新的数据
                needUpdateRuleList.add(SystemFlowRuleDTO.builderWithNodeInfo(flowId, nodeInfo));
            }else {
                // id为空或者不包含在数据库中，需要增加的数据
                needAddRuleList.add(SystemFlowRuleDTO.builderWithNodeInfo(flowId, nodeInfo));
            }
        }
        return new FlowRuleDTOPair(needUpdateRuleList, needAddRuleList);
    }

    private record FlowRuleDTOPair(List<SystemFlowRuleDTO> needUpdateRuleList, List<SystemFlowRuleDTO> needAddRuleList) {
    }

    @Override
    public List<NodeInfo> listFlowRule(String flowId) {
        if (StrUtil.isEmpty(flowId)){
            return CollUtil.newArrayList();
        }

        List<SystemFlowRule> flowRuleList = systemFlowRuleService.lambdaQuery().eq(SystemFlowRule::getFlowId, flowId).list();
        if (CollUtil.isEmpty(flowRuleList)){
            return CollUtil.newArrayList();
        }
        List<NodeInfo> nodeInfoList = flowRuleList.stream().map(flowRule -> NodeInfo.builder()
                .NodeName(flowRule.getRuleName()).id(flowRule.getId())
                .order(flowRule.getRuleOrder()).roleId(flowRule.getRoleId()).build())
                .collect(Collectors.toList());

        List<SystemFlowRuleUser> ruleUserList = systemFlowRuleUserService.lambdaQuery().in(SystemFlowRuleUser::getFlowId, flowId).list();
        Map<String, List<SystemFlowRuleUser>> ruleUserGroupByRuleId = ruleUserList.stream().collect(Collectors.groupingBy(SystemFlowRuleUser::getRuleId));
        nodeInfoList.forEach(nodeInfo -> {
            List<SystemFlowRuleUser> ruleUserListByRuleId = ruleUserGroupByRuleId.get(nodeInfo.getId());
            if (CollUtil.isNotEmpty(ruleUserListByRuleId)){
                nodeInfo.setUserIdList(ruleUserListByRuleId.stream().map(SystemFlowRuleUser::getUserId).toList());
            }
        });
        return nodeInfoList;
    }

    @Override
    public void deleteFlowRuleByRuleIds(List<String> ruleIdList) {
        if (CollUtil.isEmpty(ruleIdList)){
            log.info("流程规则id为空，不执行删除操作");
            return;
        }
        systemFlowRuleService.removeBatchByIds(ruleIdList);

        systemFlowRuleUserService.lambdaUpdate().in(SystemFlowRuleUser::getRuleId, ruleIdList).remove();
    }
}
